# Ubuntu Developer Summit web application
# Copyright (C) 2008, 2009, 2010 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.contrib.auth import logout
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext

from summit.schedule.decorators import (summit_required,
                                        summit_only_required,
                                        summit_attendee_required)

from summit.schedule.models import Summit

from summit.schedule.forms import AttendeeInterestForm

from summit.schedule.render import Schedule

__all__ = (
    'index',
    'summit',
    'interest',
    'by_date',
    'by_room',
    'csv',
)


def index(request):
    summit = Summit.objects.next()

    context = {
        'summit': summit,
    }
    return render_to_response("schedule/index.html", context,
                              context_instance=RequestContext(request))


@summit_required
def summit(request, summit, attendee):
    context = {
        'summit': summit,
        'attendee': attendee,
    }
    return render_to_response("schedule/summit.html", context,
                              context_instance=RequestContext(request))


@summit_attendee_required
def interest(request, summit, attendee):
    if request.method == 'POST':
        form = AttendeeInterestForm(request.POST, instance=attendee)
        if form.is_valid():
            form.save()
            request.user.message_set.create(
                message="Your interest has been updated.")
            return HttpResponseRedirect("/%s" % summit.name)
    else:
        form = AttendeeInterestForm(instance=attendee)

    context = {
        'summit': summit,
        'attendee': attendee,
        'form': form,
    }
    return render_to_response("schedule/interest.html", context,
                              context_instance=RequestContext(request))


@summit_required
def by_date(request, summit, attendee, date):
    schedule = Schedule(request, summit, attendee, date=date)

    if request.method == 'POST':
        return schedule.save_change()
    else:
        schedule.calculate()

    context = {
        'summit': summit,
        'attendee': attendee,
        'schedule': schedule,
        'autoreload': 'reload' in request.GET,
    }
    return render_to_response("schedule/schedule.html", context,
                              context_instance=RequestContext(request))


@summit_required
def by_room(request, summit, attendee, room_name):
    try:
        room = summit.room_set.get(tracks__title__iexact=room_name)
    except (ObjectDoesNotExist, MultipleObjectsReturned):
        room = get_object_or_404(summit.room_set, name__exact=room_name)

    schedule = Schedule(request, summit, attendee, room=room)

    if request.method == 'POST':
        return schedule.save_change()
    else:
        schedule.calculate()

    context = {
        'summit': summit,
        'attendee': attendee,
        'schedule': schedule,
        'autoreload': 'reload' in request.GET,
    }
    return render_to_response("schedule/schedule.html", context,
                              context_instance=RequestContext(request))


@summit_required
def by_track(request, summit, attendee, track_name):
    track = get_object_or_404(summit.track_set, title__iexact=track_name)

    schedule = Schedule(request, summit, attendee, track=track)

    if request.method == 'POST':
        return schedule.save_change()
    else:
        schedule.calculate()

    context = {
        'summit': summit,
        'attendee': attendee,
        'schedule': schedule,
        'autoreload': 'reload' in request.GET,
    }
    return render_to_response("schedule/schedule.html", context,
                              context_instance=RequestContext(request))


@summit_only_required
def csv(request, summit):
    schedule = Schedule(request, summit)
    schedule.calculate()

    return HttpResponse(schedule.as_csv(),
                        mimetype='text/csv')


def logout_view(request):
    logout(request)
    return HttpResponseRedirect('/')
